from PySide6 import QtCore, QtGui, QtWidgets
from PySide6.QtGui import QTextOption, QFont, QTextCursor, QTextDocument

class HTMLDelegate(QtWidgets.QStyledItemDelegate):
    def paint(self, painter, option, index):
        self.initStyleOption(option,index)

        painter.save()

        # See https://stackoverflow.com/questions/23802170/word-wrap-with-html-qtableview-and-delegates
        # and
        # https://stackoverflow.com/questions/35397943/how-to-make-a-fast-qtableview-with-html-formatted-and-clickable-cells
        doc = QtGui.QTextDocument()

        # word wrap
        #textOption = doc.defaultTextOption()
        #textOption.setWrapMode(QTextOption.WordWrap)
        #doc.setDefaultTextOption(textOption)
        #doc.setTextWidth(option.rect.width())

        doc.setHtml(option.text)

        option.text = ""
        option.widget.style().drawControl(QtWidgets.QStyle.CE_ItemViewItem, option, painter)

        painter.translate(option.rect.left(), option.rect.top())

        clip = QtCore.QRectF(0, 0, option.rect.width(), option.rect.height())
        doc.drawContents(painter, clip)

        painter.restore()


    def sizeHint(self, option, index):
        self.initStyleOption(option,index)
        doc = QtGui.QTextDocument()
        doc.setHtml(option.text)
        #doc.setTextWidth(option.rect.width())
        return QtCore.QSize(doc.idealWidth(), doc.size().height())


    def elideRichText(self, richText: str, maxWidth: int, font: QFont):
        doc = QTextDocument()
        doc.setDocumentMargin(0)
        doc.setHtml(richText)
        doc.adjustSize()
    
        if (doc.size().width() > maxWidth):
            # Elide text
            cursor = QTextCursor(doc)
            cursor.movePosition(QTextCursor.End)
    
            elidedPostfix = "..."
            metric = QFontMetrics(font)
            postfixWidth: int = metric.horizontalAdvance(elidedPostfix)
            while (doc.size().width() > maxWidth - postfixWidth):
                   cursor.deletePreviousChar()
                   doc.adjustSize()
    
            cursor.insertText(elidedPostfix)
    
            return doc.toHtml()
    
        return richText







# This Python file uses the following encoding: utf-8
import sys
import pathlib

from PySide6.QtWidgets import QApplication, QMainWindow, QFileDialog, QMessageBox, QInputDialog
from PySide6.QtCore import QFile, Slot, QDir
from PySide6.QtGui import QPalette, QColor, QColorConstants
from risicompare.form import Ui_MainWindow
from risicompare.html_delegate import HTMLDelegate
from bs4 import BeautifulSoup
import logging

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s:%(levelname)s: %(message)s'
)


class Risicompare(QMainWindow, Ui_MainWindow):
    def __init__(self):
        super(Risicompare, self).__init__()
        self.setupUi(self)

        # Connect
        self.identifierList.setItemDelegate(HTMLDelegate())
        self.bulkList.setItemDelegate(HTMLDelegate())
        self.upButton.clicked.connect(self.moveUp)
        self.downButton.clicked.connect(self.moveDown)
        self.leftButton.clicked.connect(self.moveLeft)
        self.rightButton.clicked.connect(self.moveRight)
        self.outputButton.clicked.connect(self.filePickerSave)
        self.htmlButton.clicked.connect(self.produceHTML)
        self.quitButton.triggered.connect(self.close)
        self.helpButton.triggered.connect(self.help)
        self.bulkFileButton.triggered.connect(lambda :self.filePickerOpen(True))
        self.identifierFileButton.triggered.connect(lambda: self.filePickerOpen(False))
        self.showImagesButton.triggered.connect(self.showImages)
        self.numParagraphButton.triggered.connect(self.setParagraphNumber)

        # Variables
        self.output_file = ""
        self.bulk_file = ""
        self.identifier_file = ""
        self.show_images = False
        self.part_order = dict()
        self.paragraphs = 3

        # ListView Cache delegate
        # See https://doc.qt.io/qt-6/qml-qtquick-listview.html#highlightMoveVelocity-prop
        self.identifierList.cacheBuffer = 10000
        self.bulkList.cacheBuffer = 10000


    @Slot()
    def moveUp(self):
        currentRow = self.identifierList.currentRow()
        currentItem = self.identifierList.takeItem(currentRow)
        self.identifierList.insertItem(currentRow - 1, currentItem)
        self.identifierList.setCurrentRow(currentRow - 1)


    @Slot()
    def moveDown(self):
        currentRow = self.identifierList.currentRow()
        currentItem = self.identifierList.takeItem(currentRow)
        self.identifierList.insertItem(currentRow + 1, currentItem)
        self.identifierList.setCurrentRow(currentRow + 1)


    @Slot()
    def moveLeft(self):
        currentRow = self.identifierList.currentRow()
        currentRowBulk = self.bulkList.currentRow()
        currentItem = self.identifierList.takeItem(currentRow)
        self.bulkList.insertItem(currentRowBulk + 1, currentItem)
        self.bulkList.setCurrentRow(currentRowBulk + 1)


    @Slot()
    def moveRight(self):
        currentRow = self.bulkList.currentRow()
        currentItem = self.bulkList.takeItem(currentRow)
        self.identifierList.insertItem(currentRow + 1, currentItem)
        self.identifierList.setCurrentRow(currentRow + 1)


    @Slot()
    def filePickerSave(self):
        fileName = QFileDialog.getSaveFileName(
                self,
                "Save html file",
                QDir.home().toNativeSeparators(QDir.home().path()),
                "HTML Files (*.html)"
        )
        self.output_file = fileName[0]


    @Slot(bool)
    def filePickerOpen(self, bulk):
        fileName = QFileDialog.getOpenFileName(
                self,
                "Open html file",
                QDir.home().toNativeSeparators(QDir.home().path()),
                "HTML Files (*.html)"
        )
        if bulk:
                self.bulk_file = fileName[0]
                if self.bulk_file == "":
                    return
                self.read_file(self.bulk_file, True)
        else:
                self.identifier_file = fileName[0]
                if self.identifier_file == "":
                    return
                self.read_file(self.identifier_file, False)



    @Slot()
    def read_file(self, file, bulk):
        html_list = []
        with open(file, "r") as f:
            for element in f:
                html_list.append(element)
        html = "".join(html_list)
        soup = BeautifulSoup(html, features="lxml")
        div = soup.select("div")
        # We remove some paragraph here to not
        # slow down the Delegate.
        for count , d in enumerate(div):
            p = d.select("p")
            self.part_order[count] = [div[count].decode()]
            for p in p[self.paragraphs:]:
                p.decompose()
        # The image rendering is also depressingly slow
        if not self.show_images:
            for element in div:
                for img in element.select("img"):
                    try:
                        img.attrs["src"] = ""
                    except AttributeError as e:
                        continue
        # We add the removed part into a dictionnary for
        # a lookup when writing to HTML.
        for count, element in enumerate(div):
            self.part_order[count].append(element.decode())
        if bulk:
            if self.bulkList.count() != 0:
                self.bulkList.clear()
            for element in div:
                self.bulkList.addItem(element.decode())
        else:
            if self.identifierList.count() != 0:
                self.identifierList.clear()
            for element in div:
                self.identifierList.addItem(element.decode())


    @Slot()
    def produceHTML(self):
        item_count = self.identifierList.count()
        html_risitas = []
        # We match the items html with the one in the dictionnary
        # to have a full html text and in the items order
        for element in range(item_count):
            item_to_compare = self.identifierList.item(element).text()
            for k, v in self.part_order.items():
                if item_to_compare == v[1] and item_to_compare not in html_risitas:
                    logging.info(f"The key is {k}")
                    logging.info(f"The value to compare to is '{v[1]}'")
                    logging.info(f"The element being compared is '{item_to_compare}'")
                    html_risitas.append(v[0])
                    break
        if self.output_file == "":
                msgBox = QMessageBox(icon=QMessageBox.Critical)
                msgBox.setText(f"No output file has been set!")
                msgBox.exec()
                return
        with open(self.output_file, "w") as f:
            html = (
                """<!DOCTYPE html>
                <html lang='fr'>
                <head>
                <meta charset='UTF-8'>
                <meta name='viewport' content='width=device-width, initial-scale=1.0'>
                <title>Risitas</title>
                </head>
                <body>"""
             )
            f.write(html)
            #for element in range(item_count):
            #    f.write(self.identifierList.item(element).text() + "\n")
            for element in html_risitas:
                f.write(element + "\n")
            html = (
                """</body>
                </html>"""
                )
            f.write(html)
        msgBox = QMessageBox(icon=QMessageBox.Information)
        msgBox.setText(f"The file has been written to {self.output_file}")
        msgBox.exec()



    @Slot()
    def showImages(self):
        checked = self.showImagesButton.isChecked()
        if checked:
            self.show_images = True
        else:
            self.show_images = False


    @Slot()
    def help(self):
        msgBox = QMessageBox(icon=QMessageBox.Information)
        msgBox.setText('''Scenario 1 : One chapter is missing in the identifier approach, go to left window -> select missing item
and click/press "move the item to the right button", then put the item in the right position by repeatingly pressing
the "down/up item button" for the right qlistwidget

Scenario 2: There are some duplicates in the identifier approach -> select the duplicates and click/press
the "move the item to the left button" to move them to the left QListWidget

Scenario 3: Some chapters in the identifier windows are not in the correct order -> move the items with the
move up/down buttons''')
        msgBox.exec()


    @Slot()
    def setParagraphNumber(self):
        i, ok = QInputDialog().getInt(self, "Number of paragraphs window",
                                 "Number of paragraphs to show:", 0, 0, 100, 1)
        if ok:
            self.paragraphs = i



def main():
    app = QApplication([])
    app.setStyle('Fusion')
    palette = QPalette()
    palette.setColor(QPalette.Window, QColor(53, 53, 53))
    palette.setColor(QPalette.WindowText, QColor("white"))
    palette.setColor(QPalette.Base, QColor(25, 25, 25))
    palette.setColor(QPalette.AlternateBase, QColor(53, 53, 53))
    palette.setColor(QPalette.ToolTipBase, QColor("black"))
    palette.setColor(QPalette.ToolTipText, QColor("white"))
    palette.setColor(QPalette.Text, QColor("white"))
    palette.setColor(QPalette.Button, QColor(53, 53, 53))
    palette.setColor(QPalette.ButtonText, QColor("white"))
    palette.setColor(QPalette.BrightText, QColor("red"))
    palette.setColor(QPalette.Link, QColor(42, 130, 218))
    palette.setColor(QPalette.Highlight, QColor(42, 130, 218))
    palette.setColor(QPalette.HighlightedText, QColor("black"))
    app.setPalette(palette)
    widget = Risicompare()
    widget.show()
    sys.exit(app.exec())



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'form.ui'
##
## Created by: Qt User Interface Compiler version 6.1.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import *  # type: ignore
from PySide6.QtGui import *  # type: ignore
from PySide6.QtWidgets import *  # type: ignore


class Ui_MainWindow(object):
    def setupUi(self, Risicompare):
        if not Risicompare.objectName():
            Risicompare.setObjectName(u"Risicompare")
        Risicompare.resize(1114, 801)
        self.bulkFileButton = QAction(Risicompare)
        self.bulkFileButton.setObjectName(u"bulkFileButton")
        self.identifierFileButton = QAction(Risicompare)
        self.identifierFileButton.setObjectName(u"identifierFileButton")
        self.beginningButton = QAction(Risicompare)
        self.beginningButton.setObjectName(u"beginningButton")
        self.quitButton = QAction(Risicompare)
        self.quitButton.setObjectName(u"quitButton")
        self.helpButton = QAction(Risicompare)
        self.helpButton.setObjectName(u"helpButton")
        self.showImagesButton = QAction(Risicompare)
        self.showImagesButton.setObjectName(u"showImagesButton")
        self.showImagesButton.setCheckable(True)
        self.numParagraphButton = QAction(Risicompare)
        self.numParagraphButton.setObjectName(u"numParagraphButton")
        self.centralwidget = QWidget(Risicompare)
        self.centralwidget.setObjectName(u"centralwidget")
        self.gridLayout = QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName(u"gridLayout")
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.leftButton = QPushButton(self.centralwidget)
        self.leftButton.setObjectName(u"leftButton")

        self.verticalLayout.addWidget(self.leftButton)

        self.rightButton = QPushButton(self.centralwidget)
        self.rightButton.setObjectName(u"rightButton")

        self.verticalLayout.addWidget(self.rightButton)


        self.gridLayout.addLayout(self.verticalLayout, 0, 1, 1, 1)

        self.htmlButton = QPushButton(self.centralwidget)
        self.htmlButton.setObjectName(u"htmlButton")

        self.gridLayout.addWidget(self.htmlButton, 2, 0, 1, 4)

        self.identifierList = QListWidget(self.centralwidget)
        self.identifierList.setObjectName(u"identifierList")

        self.gridLayout.addWidget(self.identifierList, 0, 2, 1, 1)

        self.bulkList = QListWidget(self.centralwidget)
        self.bulkList.setObjectName(u"bulkList")

        self.gridLayout.addWidget(self.bulkList, 0, 0, 1, 1)

        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.upButton = QPushButton(self.centralwidget)
        self.upButton.setObjectName(u"upButton")

        self.verticalLayout_2.addWidget(self.upButton)

        self.downButton = QPushButton(self.centralwidget)
        self.downButton.setObjectName(u"downButton")

        self.verticalLayout_2.addWidget(self.downButton)


        self.gridLayout.addLayout(self.verticalLayout_2, 0, 3, 1, 1)

        self.outputButton = QPushButton(self.centralwidget)
        self.outputButton.setObjectName(u"outputButton")

        self.gridLayout.addWidget(self.outputButton, 1, 0, 1, 4)

        Risicompare.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(Risicompare)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 1114, 19))
        self.menuSave = QMenu(self.menubar)
        self.menuSave.setObjectName(u"menuSave")
        self.menuHelp = QMenu(self.menubar)
        self.menuHelp.setObjectName(u"menuHelp")
        Risicompare.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(Risicompare)
        self.statusbar.setObjectName(u"statusbar")
        Risicompare.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.identifierList, self.bulkList)
        QWidget.setTabOrder(self.bulkList, self.upButton)
        QWidget.setTabOrder(self.upButton, self.downButton)
        QWidget.setTabOrder(self.downButton, self.leftButton)
        QWidget.setTabOrder(self.leftButton, self.rightButton)
        QWidget.setTabOrder(self.rightButton, self.outputButton)
        QWidget.setTabOrder(self.outputButton, self.htmlButton)

        self.menubar.addAction(self.menuSave.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())
        self.menuSave.addAction(self.bulkFileButton)
        self.menuSave.addAction(self.identifierFileButton)
        self.menuSave.addAction(self.showImagesButton)
        self.menuSave.addAction(self.numParagraphButton)
        self.menuSave.addAction(self.quitButton)
        self.menuHelp.addAction(self.helpButton)

        self.retranslateUi(Risicompare)

        QMetaObject.connectSlotsByName(Risicompare)
    # setupUi

    def retranslateUi(self, Risicompare):
        Risicompare.setWindowTitle(QCoreApplication.translate("MainWindow", u"Risicompare", None))
        self.bulkFileButton.setText(QCoreApplication.translate("MainWindow", u"Bulk file", None))
        self.identifierFileButton.setText(QCoreApplication.translate("MainWindow", u"Identifier file", None))
        self.beginningButton.setText(QCoreApplication.translate("MainWindow", u"Go back to Beginning", None))
        self.quitButton.setText(QCoreApplication.translate("MainWindow", u"Quit", None))
#if QT_CONFIG(shortcut)
        self.quitButton.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Q", None))
#endif // QT_CONFIG(shortcut)
        self.helpButton.setText(QCoreApplication.translate("MainWindow", u"Help", None))
#if QT_CONFIG(shortcut)
        self.helpButton.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+H", None))
#endif // QT_CONFIG(shortcut)
        self.showImagesButton.setText(QCoreApplication.translate("MainWindow", u"Show images", None))
        self.numParagraphButton.setText(QCoreApplication.translate("MainWindow", u"Number of  paragraph to show", None))
        self.leftButton.setText(QCoreApplication.translate("MainWindow", u"Move left", None))
#if QT_CONFIG(shortcut)
        self.leftButton.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Left", None))
#endif // QT_CONFIG(shortcut)
        self.rightButton.setText(QCoreApplication.translate("MainWindow", u"Move right", None))
#if QT_CONFIG(shortcut)
        self.rightButton.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Right", None))
#endif // QT_CONFIG(shortcut)
        self.htmlButton.setText(QCoreApplication.translate("MainWindow", u"Produce HTML", None))
#if QT_CONFIG(shortcut)
        self.htmlButton.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+D", None))
#endif // QT_CONFIG(shortcut)
        self.upButton.setText(QCoreApplication.translate("MainWindow", u"Move up", None))
#if QT_CONFIG(shortcut)
        self.upButton.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Up", None))
#endif // QT_CONFIG(shortcut)
        self.downButton.setText(QCoreApplication.translate("MainWindow", u"Move down", None))
#if QT_CONFIG(shortcut)
        self.downButton.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Down", None))
#endif // QT_CONFIG(shortcut)
        self.outputButton.setText(QCoreApplication.translate("MainWindow", u"Write HTML file to...", None))
#if QT_CONFIG(shortcut)
        self.outputButton.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+O", None))
#endif // QT_CONFIG(shortcut)
        self.menuSave.setTitle(QCoreApplication.translate("MainWindow", u"File", None))
        self.menuHelp.setTitle(QCoreApplication.translate("MainWindow", u"About", None))
    # retranslateUi

